@@ -1,12 +1,13 @@
 # vim: set expandtab ts=4 sw=4:
 
 import numpy as np
+from copy import deepcopy
 import multiprocessing as mp
 from functools import partial
-from scipy import ndimage, optimize
+from scipy import ndimage, optimize, stats
 from anamnesis import AbstractAnam, register_class
 
-from . import util
+from . import util, viz
 
 
 class AbstractModelFit(AbstractAnam):
@@ -47,6 +48,9 @@ class AbstractModelFit(AbstractAnam):
         if design is None or data_obj is None:
             return
 
+        # Store fit args or initialise an empty dict
+        self.fit_args = {} if fit_args is None else fit_args
+
         design.sanity_check()
 
         # Collapse all dimensions apart from the observations
@@ -60,6 +64,7 @@ class AbstractModelFit(AbstractAnam):
             self.is_standardised = False
 
         # Store a copy of the design matrix
+        self._design = design
         self.design_matrix = design.design_matrix
         self.regressor_list = design.regressor_list
 
@@ -75,7 +80,7 @@ class AbstractModelFit(AbstractAnam):
         self.compute_fit(design.design_matrix[self.good_observations, :],
                          data[self.good_observations, :],
                          design.contrasts,
-                         fit_args=fit_args)
+                         fit_args=self.fit_args)
 
         # Set Absolue COPES
         self.coapes = np.abs(self.copes)
@@ -142,21 +147,46 @@ class AbstractModelFit(AbstractAnam):
 
         raise NotImplementedError('This is an abstract class, please use OLSModel')
 
-    def get_prediction(self):
+    def get_prediction(self, X=None):
 
-        return self.design_matrix[self.good_observations, :].dot(self.betas)
+        if X is None:
+            X = self.design_matrix[self.good_observations, :]
+
+        betas = np.reshape(self.betas, (self.betas.shape[0], np.prod(self.betas.shape[1:])))
+
+        pred = X.dot(betas)
+
+        return np.reshape(pred, (pred.shape[0], *self.betas.shape[1:]))
 
     def get_residuals(self, data):
-
         return data - self.get_prediction()
 
+    def get_cookdist(self, data):
+        return compute_cookDistance(self.design_matrix, data)
+
+    def get_shapiro(self, data):
+
+        resids = self.get_residuals(data)
+        orig_shape = resids.shape
+
+        resids = resids.reshape(resids.shape[0], -1)
+
+        shap = np.zeros((resids.shape[1],))
+        for ii in range(len(shap)):
+            shap[ii], _ = stats.shapiro(resids[:, ii])
+
+        return shap.reshape(orig_shape[1:])
+
+    def get_studentized_residuals(self, data):
+
+        return self.get_residuals(data) / self.mse / np.sqrt(1 - self._design.leverage)[:, None]
+
     def get_resid_dots(self, data):
         resid = self.get_residuals(data)
         self.resid_dots = np.einsum('ij,ji->i', resid.T, resid)
 
-    def get_tstats(self,
-                   varcope_smoothing=None, smoothing_window=np.hanning,
-                   smooth_dims=None, sigma_hat=None):
+    def get_tstats(self, varcope_smoothing=None, smooth_dims=None,
+                   window_size=11, hat_factor=None):
         """Computes t-statistics from COPEs in a fitted model, may add optional
         temporal varcope smoothing.
 
@@ -179,8 +209,8 @@ class AbstractModelFit(AbstractAnam):
 
         """
         return get_tstats(self.copes, self.varcopes.copy(),
-                          varcope_smoothing=varcope_smoothing, smoothing_window=np.hanning,
-                          smooth_dims=None, sigma_hat=sigma_hat)
+                          varcope_smoothing=varcope_smoothing, smooth_dims=smooth_dims,
+                          window_size=window_size, hat_factor=hat_factor)
 
     def project_range(self, contrast, nsteps=2, values=None, mean_ind=0):
         """Get model prediction for a range of values across one regressor."""
@@ -241,6 +271,18 @@ class AbstractModelFit(AbstractAnam):
 
         return 1 - (self.ss_error / self.ss_total)
 
+    @property
+    def cooks_distance(self, data):
+        """https://en.wikipedia.org/wiki/Cook%27s_distance"""
+
+        raise RuntimeError
+
+        # Leverage per observation
+        hat_diag = self._design.leverage
+        term2 = hat_diag / ((1 - hat_diag)**2)
+
+        return term2
+
     @property
     def log_likelihood(self):
 
@@ -248,11 +290,15 @@ class AbstractModelFit(AbstractAnam):
 
     @property
     def aic(self):
-        return self.log_likelihood() + 2*self.num_regressors
+        # Returns the Akaike Information Criterion of model fit:
+        """www.wikipedia.com/en/Akaike_information_criterion"""
+        return 2*(self.num_regressors+1) - 2*self.log_likelihood()
 
     @property
     def bic(self):
-        return self.log_likelihood() + (self.num_regressors*np.log(self.num_observations))
+        # Returns the Bayesian Information Criterion of model fit:
+        """www.wikipedia.com/en/Bayesian_information_criterion"""
+        return (self.num_regressors+1)*np.log(self.num_observations) - 2*self.log_likelihood()
 
     @classmethod
     def load_from_hdf5(cls, hdfpath):
@@ -299,6 +345,9 @@ class AbstractModelFit(AbstractAnam):
 
 register_class(AbstractModelFit)
 
+# ------------------------------------------------------------------------
+# t-stats and corrections
+
 
 def _get_varcope_thresh2(vc, factor=2):
     from sklearn.mixture import GaussianMixture
@@ -311,24 +360,70 @@ def _get_varcope_thresh2(vc, factor=2):
     return np.exp(thresh)*factor
 
 
-def _get_varcope_thresh(vc, factor=3):
-    vc = np.log(vc.reshape(-1, 1))
-    thresh = np.max(vc)-factor
-    return np.exp(thresh)
+def varcope_corr_hat(vc, factor=1e-3, pooled_dims=None):
+    # https://www.sciencedirect.com/science/article/pii/S1053811911011906#s0010
+    if pooled_dims is None:
+        pooled_dims = np.arange(vc.ndim)
+    elif isinstance(pooled_dims, (float, int)):
+        pooled_dims = [pooled_dims]
 
+    delta = factor * vc.max(axis=tuple(pooled_dims))
+    print('Clipping varcopes at {} {}'.format(delta, delta.shape))
+
+    to_shape = np.array(vc.shape)
+    to_shape[list(pooled_dims)] = 1
+
+    vc = vc + np.broadcast_to(np.reshape(delta, to_shape), vc.shape)
 
-def varcope_hat_correction(vc, factor=3):
-    thresh = _get_varcope_thresh(vc, factor=factor)
-    vc = vc.copy()
-    vc[vc < thresh] = thresh
     return vc
 
 
+def varcope_corr_medfilt(vc, window_size=11, smooth_dims=None):
+    if smooth_dims is None:
+        smooth_dims = np.arange(vc.ndim)
+    elif isinstance(smooth_dims, (float, int)):
+        smooth_dims = [smooth_dims]
+    print('Applying medfilt smoothing of {} to dims {} of {}'.format(window_size, smooth_dims, vc.shape))
+
+    sigma = np.ones((vc.ndim,), dtype=int)
+    sigma[np.array(smooth_dims)] = window_size
+
+    return ndimage.median_filter(vc, sigma)
+
+
+def varcope_corr_avg(vc, smooth_dims=None):
+    if smooth_dims is None:
+        smooth_dims = np.arange(vc.ndim)
+    elif isinstance(smooth_dims, (float, int)):
+        smooth_dims = [smooth_dims]
+    print('Averaging varcopes over dims {}'.format(smooth_dims))
+
+    avg_vc = np.mean(vc, axis=tuple(smooth_dims))
+
+    # Broadcast back to correct shape
+    to_shape = np.array(vc.shape)
+    to_shape[list(smooth_dims)] = 1
+
+    return np.broadcast_to(np.reshape(avg_vc, to_shape), vc.shape)
+
+
+def varcope_corr_gaussfilt(vc, window_size, smooth_dims=None):
+    if smooth_dims is None:
+        smooth_dims = np.arange(vc.ndim)
+    elif isinstance(smooth_dims, (float, int)):
+        smooth_dims = [smooth_dims]
+    print('Applying gaussian smoothing of {} to dims {}'.format(window_size, smooth_dims))
+
+    sigma = np.zeros((vc.ndim,))
+    sigma[np.array(smooth_dims)] = window_size
+    return ndimage.gaussian_filter(vc, sigma)
+
+
 def get_tstats(copes, varcopes,
-               varcope_smoothing=None, smoothing_window=np.hanning,
-               smooth_dims=None, sigma_hat='auto', sigma_factor=3):
+               varcope_smoothing=None, smooth_dims=None,
+               window_size=11, hat_factor=None):
     """Computes t-statistics from COPEs in a fitted model, may add optional
-    temporal varcope smoothing.
+    varcope smoothing.
 
     Parameters
     ----------
@@ -349,45 +444,29 @@ def get_tstats(copes, varcopes,
 
     """
 
-    if sigma_hat == 'auto':
-        varcopes = varcope_hat_correction(varcopes, factor=sigma_factor)
-    elif sigma_hat is not None:
-        varcopes[varcopes < sigma_hat] = sigma_hat
+    varcopes = varcopes.copy()  # before we do any corrections...
 
-    if varcope_smoothing == 'avg':
+    if hat_factor is not None:
+        varcopes = varcope_corr_hat(varcopes, factor=hat_factor)
 
-        dim_len = varcopes.shape[smooth_dims]
-        varcopes = varcopes.mean(smooth_dims)
-        varcopes = np.expand_dims(varcopes, smooth_dims)
+    if varcope_smoothing == 'medfilt':
+        varcopes = varcope_corr_medfilt(varcopes, window_size, smooth_dims)
+    elif varcope_smoothing == 'gaussfilt':
+        varcopes = varcope_corr_gaussfilt(varcopes, window_size, smooth_dims)
+    elif varcope_smoothing == 'avg':
+        varcopes = varcope_corr_avg(varcopes, smooth_dims)
 
-        denom = np.repeat(np.sqrt(varcopes), dim_len, axis=smooth_dims)
-
-    elif varcope_smoothing is not None and varcope_smoothing > 0 and isinstance(smooth_dims, int):
-        # Create window normalised to have area of 1
-        # TODO: probably redundant with newer method below
-        w = smoothing_window(varcope_smoothing)
-        w = w / w.sum()
-
-        func = lambda m: np.convolve(m, w, mode='same')  # noqa E731
-        varcope = np.apply_along_axis(func, smooth_dims, arr=varcopes)
-
-        denom = np.sqrt(varcope)
-
-    elif varcope_smoothing is not None and len(smooth_dims) > 1:
-        sigma = np.zeros((varcopes.ndim,))
-        sigma[np.array(smooth_dims)] = varcope_smoothing
-        denom = np.sqrt(ndimage.gaussian_filter(varcopes, sigma))
-
-    else:
-        denom = np.sqrt(varcopes)
+    denom = np.sqrt(varcopes)
 
     # Compute t-values
     # run this in where to avoid RunTimeWarnings
-    tstats = np.where(np.isnan(denom) == False, copes / denom, np.nan)  # noqa E712
-
+    tstats = np.where(np.isnan(denom) == False,  # noqa: E712
+                      copes / denom,
+                      np.nan)  # noqa E712
     return tstats
 
-# -------------------------------------------------------------------------
+
+# ------------------------------------------------------------------------
 # OLS Implementation
 
 
@@ -399,13 +478,23 @@ def _get_residuals(design_matrix, betas, data):
     return data - _get_prediction(design_matrix, betas)
 
 
-def ols_fit(design_matrix, data, contrasts, method='pinv'):
+def ols_fit(design_matrix, data, contrasts, method='pinv', weights=None):
     """Fit a Ordinary Least Squares fit."""
+    from glmtools.fit import (compute_betas_pinv,
+                              compute_betas_numpy_lstsq,
+                              compute_ols_contrasts,
+                              compute_ols_varcopes)
 
     if method == 'pinv':
         betas = compute_betas_pinv(design_matrix, data)
     elif method == 'numpy_lstsq':
         betas = compute_betas_numpy_lstsq(design_matrix, data)
+    elif method == 'positive':
+        betas = compute_betas_nnls(design_matrix, data)
+    elif method == 'wols':
+        betas = compute_betas_wols(design_matrix, data, weights)
+    else:
+        print(method)
 
     copes = compute_ols_contrasts(contrasts, betas)
 
@@ -427,6 +516,30 @@ def compute_betas_numpy_lstsq(design_matrix, data):
     return b
 
 
+def compute_betas_nnls(design_matrix, data):
+
+    if data.shape[1] == 1:
+        betas = optimize.nnls(design_matrix, data)
+    else:
+        P = mp.Pool(processes=6)
+        args = [(design_matrix, data[:, ii]) for ii in range(data.shape[1])]
+        res = P.starmap(optimize.nnls, args)
+        P.close()
+
+        betas = np.vstack([res[ii][0] for ii in range(data.shape[1])]).T
+        #rnorm = np.vstack([res[ii][1] for ii in range(data.shape[1])])
+
+    return betas
+
+
+def compute_betas_wols(design_matrix, data, weights):
+    from sklearn.linear_model import LinearRegression
+
+    reg = LinearRegression().fit(design_matrix, data, sample_weight=weights)
+
+    return reg.coef_.T
+
+
 def compute_ols_contrasts(contrasts, betas):
     # Compute contrasts
     copes = contrasts.dot(betas)
@@ -460,33 +573,77 @@ class OLSModel(AbstractModelFit):
 
     def compute_fit(self, design_matrix, data, contrasts, fit_args=None):
 
-        b, c, v = ols_fit(design_matrix, data, contrasts)
+        b, c, v = ols_fit(design_matrix, data, contrasts, **fit_args)
         self.betas = b
         self.copes = c
         self.coapes = np.abs(c)
         self.varcopes = v
 
     def log_likelihood(self):
+        # Computes the total model log likehood, which is the sum of each observation's log likelihood given by:
+        # L(i) = -0.5 * log(2*pi*sigma_sq) - 0.5*(y(i)-X(i,:)B).^2/sigma_sq
+        # ll = sum(i to N) L(i)
+        #
+        # where sigma_sq is the variance of the residuals, y(i) is the ith datapoint, X(i) the ith row of
+        # the design matrix, and B the OLS coefficients.
 
         ll = - self.num_observations / 2.
-        ll = ll * np.log(self.ss_error)
-        return ll + (1 + np.log(2*np.pi)/self.num_observations)
+        ll = ll - 0.5*self.num_observations*np.log(2*np.pi*self.ss_error/self.num_observations)
+        return ll
 
 
 register_class(OLSModel)
 
+
+def run_regressor_selection(design, data, mode='forward'):
+    # Run a new model dropping each regressor in turn
+    models = [OLSModel(design, data)]
+
+    for ii in range(design.num_regressors):
+        if mode == 'forward':
+            # Keep only regressor ii
+            inds = np.setdiff1d(np.arange(design.num_regressors), ii)
+            small_design = deepcopy(design)
+            small_design.design_matrix = np.delete(small_design.design_matrix, inds, axis=1)
+            small_design.contrasts = np.delete(small_design.contrasts, inds, axis=1)
+            rname = small_design.regressor_names[ii]
+            small_design.regressor_names = [rname]
+            reg = small_design.regressor_list[ii]
+            small_design.regressor_list = [reg]
+            print("Keeping '{0}'".format(rname))
+
+            small_model = OLSModel(small_design, data)
+
+        elif mode == 'backward':
+            # Delete regressor ii
+            small_design = deepcopy(design)
+            small_design.design_matrix = np.delete(small_design.design_matrix, ii, axis=1)
+            small_design.contrasts = np.delete(small_design.contrasts, ii, axis=1)
+            rname = small_design.regressor_names.pop(ii)
+            reg = small_design.regressor_list.pop(ii)
+            print("Dropping '{0}'".format(rname))
+
+            small_model = OLSModel(small_design, data)
+
+        viz.summarise_regressor_list(small_design.regressor_list)
+        print('\n')
+        models.append(small_model)
+
+    return models
+
+
 # ---------------------------------------------------------
 # sklearn functions
 
 
-def skl_fit(design_matrix, data, contrasts, estimator=None):
+def skl_fit(design_matrix, data, contrasts, estimator=None, sample_weight=None):
     """Fit using a paramatrised SK-Learn object."""
 
     if estimator is None:
         from sklearn import linear_model
         estimator = linear_model.LinearRegression
 
-    betas, skm = _fit_sk(estimator, design_matrix, data)
+    betas, skm = _fit_sk(estimator, design_matrix, data, sample_weight=sample_weight)
 
     copes, coapes = compute_ols_contrasts(contrasts, betas)
 
@@ -552,19 +709,20 @@ class SKLModel2(AbstractModelFit):
 
     def compute_fit(self, design_matrix, data, contrasts, fit_args=None):
         from sklearn import linear_model
-        if fit_args is None:
-            skl_fitter = linear_model.LinearRegression()
-        else:
-            skl_fitter = fit_args['lm']
+        skl_fitter = fit_args.pop('fitter', None)
+        if skl_fitter is None:
+            skl_fitter = linear_model.LinearRegression(fit_intercept=False)
 
-        self.betas, self.skm = _fit_sk(skl_fitter, design_matrix, data)
+        self.betas, self.skm = _fit_sk(skl_fitter, design_matrix, data, **fit_args)
+
+        self.copes = compute_ols_contrasts(contrasts, self.betas)
 
         self.varcopes = compute_ols_varcopes(design_matrix, data, contrasts, self.betas)
 
 
-def _fit_sk(reg, design_matrix, data):
+def _fit_sk(reg, design_matrix, data, sample_weight=None):
 
-    skm = reg.fit(X=design_matrix, y=data)
+    skm = reg.fit(X=design_matrix, y=data, sample_weight=sample_weight)
     if hasattr(skm, 'coef_'):
         betas = skm.coef_.T
     elif hasattr(skm, 'estimator_') and hasattr(skm.estimator_, 'coef_'):
@@ -640,3 +798,44 @@ def flame1(design_matrix, data, S, contrasts, fixed=False, nprocesses=1):
     varcopes = np.vstack([r[2] for r in res])
 
     return betas, copes, varcopes
+
+
+def compute_cookDistance(design_matrix, data):
+    """Computes Cook distances for a given design matrix and corresponding data.
+
+        Parameters
+        ----------
+
+        design_matrix :
+            A [N x P] design matrix of regressors.
+
+        data :
+            A [N x M] vector of datapoints being analysed.
+
+        Returns
+        -------
+
+        ndarray
+            Array containing cook distances for each data sample.
+
+        """
+
+    [nsamples, ndim] = np.shape(design_matrix)
+    [nsamplesdata, ndimdata] = np.shape(data)
+
+    preds_full = np.matmul(design_matrix, np.matmul(np.linalg.pinv(design_matrix), data))
+    resid_full = data - preds_full
+    #s_sq = np.matmul(np.transpose(resid_full), resid_full) * 1 / (nsamples - ndim)
+    s_sq = np.einsum('ij,ji->i', resid_full.T, resid_full) * (1 / (nsamples - ndim))
+
+    cookdist = np.zeros([nsamples, ndimdata])
+    for i in range(nsamples):
+        print(i)
+        DMi = np.delete(design_matrix, i, 0)
+        Yi = np.delete(data, i, 0)
+        Bi = np.matmul(np.linalg.pinv(DMi), Yi)
+        pred_i = np.matmul(design_matrix, Bi)
+        sumsq = np.sum(np.square(preds_full - pred_i), axis=0)
+        cookdist[i, :] = sumsq / (s_sq * ndim)
+
+    return cookdist
