@@ -22,36 +22,42 @@ class Permutation:
     4. store null dist
     """
 
-    def __init__(self, design, data, contrast_idx, nperms, metric='copes', nprocesses=1, perm_args={}, smooth_args={}):
+    def __init__(self, design, data, contrast_idx, nperms,
+                 metric='copes', nprocesses=1, tail=0, perm_type='auto',
+                 tstat_args=None, fit_class=fit.OLSModel, verbose=False):
         from copy import deepcopy
         self._design = deepcopy(design)
         self.nperms = nperms
-        self.perm_args = perm_args
-        self.smooth_args = smooth_args
+        self.tail = tail
+        self.tstat_args = {} if tstat_args is None else tstat_args
         self.contrast_idx = contrast_idx
         self.cinds = np.where(self._design.contrasts[contrast_idx, :] != 0.)[0]
         self.cname = self._design.contrast_names[contrast_idx]
         self.nprocesses = nprocesses
+        self.fit_class = fit_class
+        self.verbose = verbose
 
+        if metric not in ['tstats', 'copes']:
+            msg = "Permutation metric must be either 'tstats' or 'copes'"
+            msg += "- ('{0}' was passed in)".format(metric)
+            raise ValueError(msg)
         self.perm_metric = metric
 
         self.rtype = self._get_rtype(contrast_idx)
         self.ctype = self._design.contrast_list[contrast_idx].ctype
 
-        self.perm_type = self._get_perm_type(self.rtype, self.ctype)
+        if perm_type == 'auto':
+            self.perm_type = self._get_perm_type(self.rtype, self.ctype)
+        else:
+            self.perm_type = perm_type
+        self.data_dims = data.data.shape
         self.data_dim_labels = data.info['dim_labels']
 
         self._print_info()
-        self.permute(data, smooth_args=self.smooth_args, **perm_args)
+        self.permute(data)
 
     def _print_info(self):
-        msg = "Permuting contrast {0} with mode={1}"
-        print(msg.format(self._design.contrast_list[self.contrast_idx], self.perm_type))
-        print("\tComputing {0} permutations".format(self.nperms))
-        if self.smooth_args.get('varcope_smoothing') is not None:
-            labels = [self.data_dim_labels[x] for x in np.atleast_1d(self.smooth_args.get('smooth_dims'))]
-            msg = "\tApplying varcope smoothing of {0} to dims {1}"
-            print(msg.format(self.smooth_args.get('varcope_smoothing'), labels))
+        _print_info(self)
 
     def _get_rtype(self, idx):
         rtype = [self._design.regressor_list[ii].rtype for ii in self.cinds]
@@ -62,7 +68,7 @@ class Permutation:
             rtype = rtype[0]
         return rtype
 
-    def _get_null_dims(self, dims, **kwargs):
+    def _get_null_dims(self, dims):
         # everything is an array for sanity
 
         # Create shape of null
@@ -70,18 +76,20 @@ class Permutation:
 
         return (self.nperms, *dims)
 
-    def _extract_perm_stat(self, null, **kwargs):
+    def _extract_perm_stat(self, null):
         'get stat out somehow'
         return null
 
-    def _compute_model_stat(self, des, data, fit=fit.OLSModel, smooth_args={}):
+    def _compute_model_stat(self, des, data):
 
-        f = fit(des, data)
-        if smooth_args.get('varcope_smoothing') is None:
-            return getattr(f, self.perm_metric)[self.contrast_idx, ...]
+        f = self.fit_class(des, data)
+        if self.perm_metric == 'tstats':
+            # Use defaults
+            ret = f.get_tstats(**self.tstat_args)
         else:
-            tstats = f.get_tstats(**smooth_args)
-            return tstats[self.contrast_idx, ...]
+            ret = getattr(f, self.perm_metric)
+
+        return ret[self.contrast_idx, ...]
 
     def _get_perm_type(self, rtype, ctype):
         'sign-flip etc'
@@ -101,12 +109,12 @@ class Permutation:
         else:
             return mode
 
-    def _run_perm(self, data, first, fit, smooth_args, perm_stat_args):
+    def _run_perm(self, data, first):
         """
         data
         first = False
         fit = fit.OLSModel
-        smooth_args = {}
+        tstat_args = {}
         perm_stat_args={}
         """
 
@@ -114,49 +122,35 @@ class Permutation:
 
         if first:
             # Don't permute design matrix
-            perm = self._compute_model_stat(self._design, data, fit=fit, smooth_args=smooth_args)
+            perm = self._compute_model_stat(self._design, data)
         else:
             perm_design = self.permute_design_matrix()
-            perm = self._compute_model_stat(perm_design, data, fit=fit, smooth_args=smooth_args)
+            perm = self._compute_model_stat(perm_design, data)
 
-        perm_stat = self._extract_perm_stat(perm, **perm_stat_args)
+        perm_stat = self._extract_perm_stat(perm)
         return perm_stat
 
     def permute_design_matrix(self):
 
-        X = self._design.design_matrix.copy()
-        if self.perm_type == 'sign-flip':
-            perm_inds = np.random.permutation(np.tile([1, -1], int(X.shape[0]/2)))
-            if np.remainder(X.shape[0], 2) == 1:
-                perm_inds = np.r_[perm_inds, np.random.choice([1, -1])]
-            X[:, self.cinds] = X[:, self.cinds] * perm_inds[:, None]
-        elif self.perm_type == 'row-shuffle':
-            perm_inds = np.random.permutation(X.shape[0])
-            ix = np.ix_(perm_inds, self.cinds)  # np.ix allows us to apply indexing to both dims
-            X[:, self.cinds] = X[ix]
+        return permute_design_matrix(self._design, self.cinds, self.perm_type)
 
-        perm_design = deepcopy(self._design)
-        perm_design.design_matrix = X
+    def permute(self, data):
 
-        return perm_design
-
-    def permute(self, data, fit=fit.OLSModel, smooth_args={}, **kwargs):
-
-        null_dims = self._get_null_dims(data.data.shape, **kwargs)
+        null_dims = self._get_null_dims(data.data.shape)
         nulls = np.zeros(null_dims)
-        nulls[0, ...] = self._run_perm(data, first=True, fit=fit,
-                                       smooth_args=smooth_args,
-                                       perm_stat_args=kwargs)
+        nulls[0, ...] = self._run_perm(data, first=True)
 
         #for ii in range(1, self.nperms):
         #    nulls[ii, ...] = self._run_perm(data, first=False, fit=fit,
-        #                                    smooth_args=smooth_args,
+        #                                    tstat_args=tstat_args,
         #                                    perm_stat_args=kwargs)
 
         import multiprocessing as mp
-        p = mp.Pool(processes=self.nprocesses)
-        args = [(data, False, fit, smooth_args, kwargs) for ii in range(1, self.nperms)]
-        res = p.starmap(self._run_perm, args)
+        #p = mp.Pool(processes=self.nprocesses)
+        args = [(data, False) for ii in range(1, self.nperms)]
+
+        with mp.Pool(processes=self.nprocesses) as p:
+            res = p.starmap(self._run_perm, args)
 
         nulls[1:, ...] = np.array(res)
 
@@ -169,25 +163,45 @@ class Permutation:
         else:
             return np.nanpercentile(self.nulls, percentiles, axis=0)
 
-    def get_sig_at_percentile(self, percentile):
+    def get_sig_at_percentile(self, percentile, tail=0):
 
         thresh = self.get_thresh(percentile)
 
-        return self.nulls[0, ...] > thresh
+        if tail == 1:
+            return self.nulls[0, ...] > thresh
+        elif tail == -1:
+            return self.nulls[0, ...] < thresh
+        elif tail == 0:
+            return np.abs(self.nulls[0, ...]) > thresh
+
+
+def _print_info(P):
+    msg = "Permuting contrast {0} with mode={1}"
+    print(msg.format(P._design.contrast_list[P.contrast_idx], P.perm_type))
+    print("\tComputing {0} permutations".format(P.nperms))
+    if P.tstat_args.get('varcope_smoothing') is not None:
+        labels = [P.data_dim_labels[x] for x in np.atleast_1d(P.tstat_args.get('smooth_dims'))]
+        msg = "\tApplying varcope smoothing of {0} to dims {1}"
+        print(msg.format(P.tstat_args.get('varcope_smoothing'), labels))
 
 
 def permute_design_matrix(design, cinds, perm_type):
 
     X = design.design_matrix.copy()
     if perm_type == 'sign-flip':
-        perm_inds = np.random.permutation(np.tile([1, -1], int(X.shape[0]/2)))
-        if np.remainder(X.shape[0], 2) == 1:
-            perm_inds = np.r_[perm_inds, np.random.choice([1, -1])]
+        #perm_inds = np.random.permutation(np.tile([1, -1], int(X.shape[0]/2)))
+        #if np.remainder(X.shape[0], 2) == 1:
+        #    perm_inds = np.r_[perm_inds, np.random.choice([1, -1])]
+        perm_inds = np.random.choice([1, -1], X.shape[0])
         X[:, cinds] = X[:, cinds] * perm_inds[:, None]
     elif perm_type == 'row-shuffle':
         perm_inds = np.random.permutation(X.shape[0])
         ix = np.ix_(perm_inds, cinds)  # np.ix allows us to apply indexing to both dims
         X[:, cinds] = X[ix]
+    elif perm_type == 'roll':
+        roll_len = np.random.randint(0, X.shape[0])
+        for idx in cinds:
+            X[:, idx] = np.roll(X[:, idx], roll_len)
 
     perm_design = deepcopy(design)
     perm_design.design_matrix = X
@@ -197,9 +211,13 @@ def permute_design_matrix(design, cinds, perm_type):
 
 class MaxStatPermutation(Permutation):
 
-    def _get_null_dims(self, dims, **kwargs):
+    def __init__(self, *args, **kwargs):
+        self.pooled_dims = kwargs.pop('pooled_dims', [])
+        super().__init__(*args, **kwargs)
+
+    def _get_null_dims(self, dims):
         # everything is an array for sanity
-        self.pooled_dims = kwargs.get('pooled_dims', [])
+        self.pooled_dims = self.pooled_dims
 
         dims = np.atleast_1d(dims)
         dim_range = np.array(np.arange(len(dims)))
@@ -215,7 +233,7 @@ class MaxStatPermutation(Permutation):
 
         return null_dims
 
-    def _extract_perm_stat(self, null, **kwargs):
+    def _extract_perm_stat(self, null):
 
         # Adjust as contrasts dim has been removed
         pd = np.array(self.pooled_dims) - 1
@@ -229,10 +247,20 @@ class MaxStatPermutation(Permutation):
 
 class ClusterPermutation(Permutation):
 
-    def _get_null_dims(self, dims, **kwargs):
-        # everything is an array for sanity
-        self.pooled_dims = kwargs.get('pooled_dims', [])
+    def __init__(self, *args, **kwargs):
+        self.pooled_dims = kwargs.pop('pooled_dims', [])
+        self.stat_power = kwargs.pop('stat_power', 1)
+        self.cluster_forming_threshold = kwargs.pop('cluster_forming_threshold', [])
+        super().__init__(*args, **kwargs)
 
+    def _print_info(self):
+        super()._print_info()
+
+        labels = [self.data_dim_labels[x] for x in np.atleast_1d(self.pooled_dims)]
+        print('\tFinding clusters in {0} dimensions'.format(labels))
+
+    def _get_null_dims(self, dims):
+        # everything is an array for sanity
         dims = np.atleast_1d(dims)
         dim_range = np.array(np.arange(len(dims)))
 
@@ -242,9 +270,6 @@ class ClusterPermutation(Permutation):
         # Create shape of null
         null_dims = (self.nperms, *dims[self.nonpooled_dims])
 
-        labels = [self.data_dim_labels[x] for x in np.atleast_1d(self.pooled_dims)]
-        print('\tFinding clusters in {0} dimensions'.format(labels))
-
         return null_dims
 
     def _extract_clusters(self, null):
@@ -252,21 +277,20 @@ class ClusterPermutation(Permutation):
         Return the cluster masks and values of clusters in a dataset
         """
 
-        cluster_forming_threshold = self.perm_args.get('cluster_forming_threshold')
-        stat_power = self.perm_args.get('stat_power', 1)
-
         nonpooled_dims = np.array(self.nonpooled_dims) - 1  # adjust as contrasts dim has been removed
-        cluster_masks, cluster_stats = _find_clusters(null, cluster_forming_threshold,
+        cluster_masks, cluster_stats = _find_clusters(null,
+                                                      self.cluster_forming_threshold,
                                                       nonpooled_dims,
-                                                      stat_power)
+                                                      stat_power=self.stat_power,
+                                                      tail=self.tail)
 
         return cluster_masks, cluster_stats
 
-    def _extract_perm_stat(self, null, ret_clusters=False, **kwargs):
+    def _extract_perm_stat(self, null, ret_clusters=False):
         """
         Return value of largest cluster in each of the nonpooled dimensions
-        """
 
+        """
         cluster_masks, cluster_stats = self._extract_clusters(null)
         pooled_dims = np.array(self.pooled_dims) - 1
 
@@ -275,7 +299,13 @@ class ClusterPermutation(Permutation):
             cluster_values[cluster_masks == c + 1] = cluster_stats[c]
 
         # Largest cluster value for each of the nonpooled dimensions
-        cluster_max_per_dim = np.max(cluster_values, axis=tuple(pooled_dims))
+        if self.tail == 1:
+            cluster_max_per_dim = np.max(cluster_values, axis=tuple(pooled_dims))
+        elif self.tail == -1:
+            # Take the abs of lower end here - not sure if that is right...
+            cluster_max_per_dim = np.abs(np.min(cluster_values, axis=tuple(pooled_dims)))
+        elif self.tail == 0:
+            cluster_max_per_dim = np.max(np.abs(cluster_values), axis=tuple(pooled_dims))
 
         return cluster_max_per_dim
 
@@ -310,20 +340,25 @@ class ClusterPermutation(Permutation):
         else:
             return cluster_stats_nonpooled
 
-    def get_obs_clusters(self, data, fit=fit.OLSModel):
+    def get_obs_clusters(self, data):
 
-        f = fit(self._design, data)
-        metric = getattr(f, self.perm_metric)[self.contrast_idx, ...]
+        f = self.fit_class(self._design, data)
+        if len(self.tstat_args) != 0 and self.perm_metric == 'tstats':
+            # Use defaults
+            metric = f.get_tstats(**self.tstat_args)
+        else:
+            metric = getattr(f, self.perm_metric)
+        metric = metric[self.contrast_idx, ...]
 
         cluster_masks, cluster_stats = self._extract_clusters(metric)
 
         return cluster_masks, cluster_stats
 
-    def get_sig_clusters(self, data, thresh, fit=fit.OLSModel):
+    def get_sig_clusters(self, data, thresh):
 
         pooled_dims = np.array(self.pooled_dims) - 1
 
-        c = self.get_obs_clusters(data, fit=fit)
+        c = self.get_obs_clusters(data)
         cluster_masks, cluster_stats = c
 
         if cluster_masks is None:
@@ -333,9 +368,28 @@ class ClusterPermutation(Permutation):
 
         sig_inds = []
         for c in range(len(cluster_stats)):
-            clust_nonpooled_inds = np.sum(cluster_masks == c + 1, axis=tuple(pooled_dims)) > 0
-            if cluster_stats[c] > thresh[clust_nonpooled_inds]:
-                sig_inds.append(c)
+            clust_nonpooled_inds = np.sum(cluster_masks == c + 1,
+                                          axis=tuple(pooled_dims)) > 0
+            if isinstance(thresh, (int, float)):
+                if self.tail == 1:
+                    if cluster_stats[c] > thresh:
+                        sig_inds.append(c)
+                elif self.tail == -1:
+                    if cluster_stats[c] < -thresh:
+                        sig_inds.append(c)
+                elif self.tail == 0:
+                    if np.abs(cluster_stats[c]) > thresh:
+                        sig_inds.append(c)
+            else:
+                if self.tail == 1:
+                    if cluster_stats[c] > thresh[clust_nonpooled_inds]:
+                        sig_inds.append(c)
+                elif self.tail == -1:
+                    if cluster_stats[c] < thresh[clust_nonpooled_inds]:
+                        sig_inds.append(c)
+                elif self.tail == 0:
+                    if np.abs(cluster_stats[c]) > thresh[clust_nonpooled_inds]:
+                        sig_inds.append(c)
 
         if len(sig_inds) > 0:
             sig_cluster_masks = np.zeros_like(cluster_masks)
@@ -367,13 +421,18 @@ def _define_connectivity_structure(rank, nonpooled_dims):
     return np.add.reduce(q, 0) <= connectivity_dist
 
 
-def _find_clusters(metric, cluster_forming_threshold, nonpooled_dims=[], stat_power=0):
+def _find_clusters(metric, cluster_forming_threshold, nonpooled_dims=[], stat_power=1, tail=0):
 
     # Get structure connectivity based on pooled dims
     conn = _define_connectivity_structure(metric.ndim, nonpooled_dims)
 
     # Find some clusters
-    labels, num_features = ndimage.label(metric**stat_power > cluster_forming_threshold, conn)
+    if tail == 1:
+        labels, num_features = ndimage.label(metric**stat_power > cluster_forming_threshold, conn)
+    elif tail == -1:
+        labels, num_features = ndimage.label(metric**stat_power < cluster_forming_threshold, conn)
+    elif tail == 0:
+        labels, num_features = ndimage.label(np.abs(metric**stat_power) > cluster_forming_threshold, conn)
 
     # Stop here if no clusters are  found
     if num_features == 0:
@@ -512,7 +571,24 @@ def get_sensor_time_clusters(X, A, cft=5, pwr=1):
     return C3, cinds, cexts
 
 
-class MNEClusterPermutation3(Permutation):
+class MNEClusterPermutation(Permutation):
+
+    def __init__(self, *args, **kwargs):
+        self.cluster_forming_threshold = kwargs.pop('cluster_forming_threshold', [])
+        self.adjacency = kwargs.pop('adjacency', [])
+        super().__init__(*args, **kwargs)
+
+    def _print_info(self):
+        super()._print_info()
+
+        msg = '\tComputing clusters across {0} time-points and {1} channels'
+        print(msg.format(self.data_dims[1], self.data_dims[2]))
+
+        msg = '\tUsing cluster forming threshold: {0}'.format(self.cluster_forming_threshold)
+        print(msg)
+
+        if (self.adjacency is None) or (len(self.adjacency) == 0):
+            print("\tNO ADJACENCY INFO FOUND? - check inputs")
 
     def _get_null_dims(self, dims, **kwargs):
         return (self.nperms,)
@@ -520,19 +596,18 @@ class MNEClusterPermutation3(Permutation):
     def _extract_perm_stat(self, null, **kwargs):
         from mne.stats.cluster_level import _find_clusters as mne_find_clusters
 
-        threshold = self.perm_args.get('cluster_forming_threshold')
-        adjacency = self.perm_args.get('adjacency', None)
-
         flatt = null.flatten()
         clus, cstats = mne_find_clusters(flatt,
-                                         threshold,
-                                         adjacency=adjacency)
+                                         self.cluster_forming_threshold,
+                                         adjacency=self.adjacency)
 
         if len(clus) == 0:
-            print('No clusters')
+            if self.verbose:
+                print('No clusters')
             return 0
 
-        print('Found {0} clusters - {1} is largest'.format(len(cstats), np.abs(cstats).max()))
+        if self.verbose:
+            print('Found {0} clusters - {1} is largest'.format(len(cstats), np.abs(cstats).max()))
 
         if len(cstats) == 0:
             return 0
@@ -544,15 +619,15 @@ class MNEClusterPermutation3(Permutation):
         from mne.stats.cluster_level import _reshape_clusters as mne_reshape_clusters
 
         f = fit(self._design, data)
-        obs = getattr(f, self.perm_metric)[self.contrast_idx, ...]
-
-        threshold = self.perm_args.get('cluster_forming_threshold')
-        adjacency = self.perm_args.get('adjacency')
+        if self.perm_metric == 'copes':
+            obs = getattr(f, self.perm_metric)[self.contrast_idx, ...]
+        elif self.perm_metric == 'tstats':
+            obs = f.get_tstats(**self.tstat_args)[self.contrast_idx, ...]
 
         flatt = obs.flatten()
         clus, cstats = mne_find_clusters(flatt,
-                                         threshold,
-                                         adjacency=adjacency)
+                                         self.cluster_forming_threshold,
+                                         adjacency=self.adjacency)
 
         clus = mne_reshape_clusters(clus, obs.shape)
 
@@ -579,89 +654,6 @@ class MNEClusterPermutation3(Permutation):
         return out, obs
 
 
-class MNEClusterPermutation2(Permutation):
-
-    def _get_null_dims(self, dims, **kwargs):
-        return (self.nperms,)
-
-    def _extract_perm_stat(self, null, **kwargs):
-
-        threshold = self.perm_args.get('cluster_forming_threshold')
-        connectivity = self.perm_args.get('connectivity')
-
-        clusters, cinds, cstats = get_sensor_time_clusters(null, connectivity, cft=threshold, pwr=1)
-
-        if (len(cstats) == 1) and (cinds[0] == -1):
-            print('No clusters')
-            return 0
-        else:
-            goods = np.where(cinds > -1)[0]
-            cstats = cstats[goods]
-
-        print('Found {0} clusters - {1} is largest'.format(len(cstats), cstats.max()))
-
-        if len(cstats) == 0:
-            return 0
-        else:
-            return cstats.max()
-
-    def get_obs_clusters(self, data, fit=fit.OLSModel):
-
-        f = fit(self._design, data)
-        obs = getattr(f, self.perm_metric)[self.contrast_idx, ...]
-
-        threshold = self.perm_args.get('cluster_forming_threshold')
-        connectivity = self.perm_args.get('connectivity')
-
-        clusters, cinds, cstats = get_sensor_time_clusters(obs, connectivity, cft=threshold)
-
-        return obs, clusters, cstats
-
-
-class MNEClusterPermutation(Permutation):
-
-    def _get_null_dims(self, dims, **kwargs):
-        return (self.nperms,)
-
-    def _extract_perm_stat(self, null, **kwargs):
-        from mne.stats.cluster_level import _find_clusters as mne_find_clusters
-
-        threshold = self.perm_args.get('cluster_forming_threshold')
-        connectivity = self.perm_args.get('connectivity')
-
-        clus, cstat = mne_find_clusters(null.flatten(),
-                                        threshold=threshold,
-                                        connectivity=connectivity)
-
-        if len(cstat) == 0:
-            return 0
-        else:
-            return cstat.max()
-
-    def get_obs_clusters(self, data, fit=fit.OLSModel):
-
-        from mne.stats.cluster_level import _find_clusters as mne_find_clusters
-        from mne.stats.cluster_level import _reshape_clusters as mne_reshape_clusters
-
-        f = fit(self._design, data)
-        obs = getattr(f, self.perm_metric)[self.contrast_idx, ...]
-
-        threshold = self.perm_args.get('cluster_forming_threshold')
-        connectivity = self.perm_args.get('connectivity')
-
-        clus, cstat = mne_find_clusters(obs.flatten(),
-                                        threshold=threshold,
-                                        connectivity=connectivity)
-
-        clusters = mne_reshape_clusters(clus, obs.shape)
-
-        cluster_masks = np.zeros((len(clusters), *obs.shape))
-        for ii in range(len(clusters)):
-            cluster_masks[ii, clusters[ii][0], clusters[ii][1]] = 1
-
-        return clusters, cluster_masks, cstat
-
-
 # -----------------------------------------------
 
 
@@ -842,27 +834,83 @@ def _check_false_positives(nperms=2000):
 
     model = fit.OLSModel(des, dat)
 
-    nulls = permute_glm(des, dat, 2000)
-    thresh = np.percentile(nulls, [95, 99, 99.9])
+    P = Permutation(des, dat, 0, nperms,
+                    metric='tstats', nprocesses=6)
+
+    thresh = P.get_thresh([95, 99, 99.9])
 
     msg = '{0}/{1} - {2}%'
 
     print('False positives at alpha=.95')
-    pc = (np.sum(model.copes > thresh[0]) / np.product(model.copes.shape)) * 100
-    print(msg.format(np.sum(model.copes > thresh[0]),
-                     np.product(model.copes.shape),
+    pc = (np.sum(model.tstats > thresh[0]) / np.product(model.tstats.shape)) * 100
+    print(msg.format(np.sum(model.tstats > thresh[0]),
+                     np.product(model.tstats.shape),
                      pc))
 
     print('False positives at alpha=.99')
-    pc = (np.sum(model.copes > thresh[1]) / np.product(model.copes.shape)) * 100
-    print(msg.format(np.sum(model.copes > thresh[1]),
-                     np.product(model.copes.shape),
+    pc = (np.sum(model.tstats > thresh[1]) / np.product(model.tstats.shape)) * 100
+    print(msg.format(np.sum(model.tstats > thresh[1]),
+                     np.product(model.tstats.shape),
                      pc))
 
     print('False positives at alpha=.999')
-    pc = (np.sum(model.copes > thresh[2]) / np.product(model.copes.shape)) * 100
-    print(msg.format(np.sum(model.copes > thresh[2]),
-                     np.product(model.copes.shape),
+    pc = (np.sum(model.tstats > thresh[2]) / np.product(model.tstats.shape)) * 100
+    print(msg.format(np.sum(model.tstats > thresh[2]),
+                     np.product(model.tstats.shape),
+                     pc))
+
+
+def _check_false_positives_maxstat(nperms=2000):
+    """ Utility script for a simple false positive rate check.
+
+    This is very noisy so isn't included as a formal test case but is useful to
+    make sure that a simple white noise test is giving approximately correct
+    false positive rates.  """
+    from . import design, data
+
+    A = np.random.randn(100, 500, 5)
+    B = np.random.randn(100, 500, 5)
+
+    X = np.r_[A, B]
+    categories = np.repeat((1, 2), 100)
+    dat = data.TrialGLMData(data=X,
+                            category_list=categories,
+                            dim_labels=['Trials', 'Voxels', 'Frequencies'])
+
+    dat.dim_labels = dat.info['dim_labels']
+
+    DC = design.DesignConfig()
+    DC.add_regressor(name='A', rtype='Categorical', codes=1)
+    DC.add_regressor(name='B', rtype='Categorical', codes=2)
+    DC.add_contrast(name='GroupDiff', values=[1, -1])
+
+    des = DC.design_from_datainfo(dat.info)
+
+    model = fit.OLSModel(des, dat)
+
+    P = MaxStatPermutation(des, dat, 0, nperms, pooled_dims=(1, 2),
+                           metric='tstats', nprocesses=6)
+
+    thresh = P.get_thresh([95, 99, 99.9])
+
+    msg = '{0}/{1} - {2}%'
+
+    print('False positives at alpha=.95')
+    pc = (np.sum(model.tstats > thresh[0]) / np.product(model.tstats.shape)) * 100
+    print(msg.format(np.sum(model.tstats > thresh[0]),
+                     np.product(model.tstats.shape),
+                     pc))
+
+    print('False positives at alpha=.99')
+    pc = (np.sum(model.tstats > thresh[1]) / np.product(model.tstats.shape)) * 100
+    print(msg.format(np.sum(model.tstats > thresh[1]),
+                     np.product(model.tstats.shape),
+                     pc))
+
+    print('False positives at alpha=.999')
+    pc = (np.sum(model.tstats > thresh[2]) / np.product(model.tstats.shape)) * 100
+    print(msg.format(np.sum(model.tstats > thresh[2]),
+                     np.product(model.tstats.shape),
                      pc))
 
 
