@@ -19,17 +19,16 @@ class AbstractRegressor(AbstractAnam):
     def __str__(self):
         return "%s(%s,%s)" % (self.__class__, self.name, self.rtype)
 
-    hdf5_outputs = ['rtype', 'values', 'name']
+    hdf5_outputs = ['rtype', 'values', 'name', 'values_orig']
 
     def __init__(self, **kwargs):
         AbstractAnam.__init__(self)
 
         self.rtype = None
         self.values = None
+        self.values_orig = None
         self.name = None
 
-        self.parse_arguments(kwargs)
-
     def parse_arguments(self, kwargs):
         raise NotImplementedError('This is an AbtractClass')
 
@@ -49,7 +48,8 @@ class AbstractRegressor(AbstractAnam):
 
 class ConstantRegressor(AbstractRegressor):
 
-    def __init__(self, **kwargs):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         AbstractAnam.__init__(self)
 
         self.rtype = 'Constant'
@@ -89,7 +89,8 @@ register_class(ConstantRegressor)
 
 class TrendRegressor(AbstractRegressor):
 
-    def __init__(self, **kwargs):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         AbstractAnam.__init__(self)
 
         self.rtype = 'Trend'
@@ -136,9 +137,67 @@ class TrendRegressor(AbstractRegressor):
 register_class(TrendRegressor)
 
 
+class SineRegressor(AbstractRegressor):
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        AbstractAnam.__init__(self)
+
+        self.rtype = 'Sine'
+
+        if len(kwargs) > 0:
+            self.parse_arguments(kwargs)
+
+            t = np.linspace(0, 1, self.num_observations)
+            self.values = np.sin(2*np.pi*self.freq*t + self.phase)
+            self.normalise_values()
+        else:
+            self.values = None
+            self.name = 'Unknown'
+
+    def parse_arguments(self, extraargs):
+        """
+        num_observations
+        power
+        preproc
+        """
+
+        if 'num_observations' in extraargs:
+            self.num_observations = extraargs['num_observations']
+        else:
+            raise ValueError('num_observations not passed to ConditionRegressor')
+
+        if 'freq' in extraargs:
+            self.freq = extraargs['freq']
+        else:
+            # Default is a linear trend
+            self.freq = 1
+
+        if 'phase' in extraargs:
+            self.phase = extraargs['phase']
+        else:
+            # Default is a linear trend
+            self.phase = 0
+
+        if 'name' in extraargs:
+            self.name = extraargs['name']
+        else:
+            self.name = 'Condition'
+
+        if 'preproc' in extraargs:
+            self.preproc = extraargs['preproc']
+        else:
+            # Default to unit maximum normalisation
+            self.preproc = 'unitmax'
+
+
+register_class(SineRegressor)
+
+
 class CategoricalRegressor(AbstractRegressor):
 
-    def __init__(self, **kwargs):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         AbstractAnam.__init__(self)
 
         self.rtype = 'Categorical'
@@ -199,7 +258,8 @@ register_class(CategoricalRegressor)
 
 class ParametricRegressor(AbstractRegressor):
 
-    def __init__(self, **kwargs):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         AbstractAnam.__init__(self)
 
         self.rtype = 'Parametric'
@@ -250,9 +310,55 @@ class ParametricRegressor(AbstractRegressor):
 register_class(ParametricRegressor)
 
 
+class InteractionRegressor(AbstractRegressor):
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        AbstractAnam.__init__(self)
+
+        self.rtype = 'Interaction'
+
+        if len(kwargs) > 0:
+            self.parse_arguments(kwargs)
+            self.normalise_values()
+        else:
+            self.values = None
+            self.name = 'Unknown'
+
+    def parse_arguments(self, extraargs):
+        """
+        values
+        preproc
+        """
+
+        if 'regressors' in extraargs:
+            vals = np.hstack([np.array(extraargs[key])[:, None] for key in extraargs['regressors']])
+            self.values = np.prod(vals, axis=1)
+        else:
+            raise ValueError('"regressors" not passed to InteractionRegressor class')
+
+        if self.values.shape[0] != extraargs['num_observations']:
+            msg = 'Regressor shape ({0}) and Data shape ({1}) do not match'
+            raise ValueError(msg.format(self.values.shape, extraargs['num_observations']))
+
+        if 'preproc' in extraargs:
+            self.preproc = extraargs['preproc']
+        else:
+            # Default is to z-transform values
+            self.preproc = 'z'
+
+        if 'name' in extraargs:
+            self.name = extraargs['name']
+        else:
+            self.name = 'Interaction'
+
+
+register_class(InteractionRegressor)
+
+
 class BoxcarRegressor(AbstractRegressor):
 
-    def __init__(self, **kwargs):
+    def __init__(self, *args, **kwargs):
         """Create a boxcar regressor
 
         Parameters
@@ -275,6 +381,7 @@ class BoxcarRegressor(AbstractRegressor):
         BoxcarRegressor instance
 
         """
+        super().__init__(*args, **kwargs)
         AbstractAnam.__init__(self)
 
         self.rtype = 'Boxcar'
@@ -324,7 +431,8 @@ register_class(BoxcarRegressor)
 
 class ConvolutionalRegressor(AbstractRegressor):
 
-    def __init__(self, **kwargs):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
         AbstractAnam.__init__(self)
 
         self.rtype = 'Convolutional'
