@@ -149,16 +149,30 @@ def standardise_data(X):
 # Design helpers
 
 
-def ftests_from_dict(config):
+def ftests_from_dict(config, regs=None):
 
     ftests = list()
     for con in config:
-        vals = np.array(con['values'].split()).astype(float)
+        if isinstance(con['values'], str):
+            vals = np.array(con['values'].split()).astype(float)
+        elif isinstance(con['values'], int):
+            vals = [con['values']]
+        elif isinstance(con['values'], list):
+            vals = con['values']
+        elif isinstance(con['values'], tuple):
+            vals = list(con['values'])
+        elif isinstance(con['values'], dict):
+            if regs is None:
+                return ValueError('Missing contrast info, cannot define f-test')
+            vals = np.zeros(len(regs),)
+            rnames = [r.name for r in regs]
+            for key in con['values'].keys():
+                vals[rnames.index(key)] = con['values'][key]
         ftests.append(design.FTest(name=con['name'], values=vals))
 
     # Sanity check
     if len(set([len(a.values) for a in ftests])) > 1:
-        raise ValueError('F-test lengths are mismatched')
+        raise ValueError('FTest lengths are mismatched')
 
     return ftests
 
@@ -406,6 +420,10 @@ def regressors_from_datainfo(reg_conf, data_info):
                                                       category_list=groups)
                 data_regressors.append(reg)
 
+        elif reg_type == 'Interaction':
+            print(reg)
+            reg = regressors.InteractionRegressor()
+
         else:
             r = getattr(regressors, reg_type)
 
